#include "driver/i2s.h"
#include "driver/uart.h"
#include "audio_data.h"

#define I2S_NUM         I2S_NUM_0 // I2S port number
#define I2S_BCK_IO      (27)      // BCK pin
#define I2S_WS_IO       (25)      // WS pin
#define I2S_DO_IO       (26)      // SD_OUT pin
#define I2S_DI_IO       (32)      // SD_IN pin

#define UART_NUM        UART_NUM_1 // UART port number
#define TXD_PIN         (17)       // UART TX pin
#define RXD_PIN         (33)       // UART RX pin
#define BUF_SIZE        (1024)     // UART buffer size
#define UART_BUF_SIZE   1024

unsigned long lastTime = 0;
const int beepFrequency = 1000; // Frequency of the beep in Hz
const int beepDuration = 100;    // Duration of the beep in milliseconds
const int beepSampleRate = 8000; // Sample rate for beep generation
const int beepVolume = 500;      // Volume of the beep

#define SAMPLE_RATE 8000
#define TONE_FREQ 5000   // Frequency in Hz (A4 note)
#define DURATION 2      // Duration in seconds

// SMS details
const char* phoneNumber = "+989212041856"; // Replace with recipient's phone number
const char* messageText = "salam\x1A"; // Replace with your message


const int numSamples = SAMPLE_RATE * DURATION;
int8_t wavData[numSamples];

void generateTone() {
    for (int i = 0; i < numSamples; ++i) {
        // Generate a sine wave at the specified frequency
        wavData[i] = (int8_t)(127 * sin(2 * PI * TONE_FREQ * i / SAMPLE_RATE));
    }
}

void setup() {
    Serial.begin(115200);
    //generateTone(); 
    Serial.print("Initializing...");
    // Initialize I2S
    i2s_config_t i2s_config = {
        .mode = (i2s_mode_t)(I2S_MODE_MASTER | I2S_MODE_RX | I2S_MODE_TX), // Cast to i2s_mode_t
        .sample_rate = 8000,
        .bits_per_sample = I2S_BITS_PER_SAMPLE_16BIT,
        .channel_format = I2S_CHANNEL_FMT_RIGHT_LEFT, // Make sure this is in the right order
        .communication_format = I2S_COMM_FORMAT_I2S,
        .intr_alloc_flags = 0,
        .dma_buf_count = 8,
        .dma_buf_len = 64
    };
    i2s_pin_config_t pin_config = {
        .bck_io_num = I2S_BCK_IO,
        .ws_io_num = I2S_WS_IO,
        .data_out_num = I2S_DO_IO,
        .data_in_num = I2S_DI_IO,
    };
    i2s_driver_install(I2S_NUM, &i2s_config, 0, NULL);
    i2s_set_pin(I2S_NUM, &pin_config);

    // Initialize UART
    uart_config_t uart_config = {
        .baud_rate = 115200,
        .data_bits = UART_DATA_8_BITS,
        .parity = UART_PARITY_DISABLE,
        .stop_bits = UART_STOP_BITS_1,
        .flow_ctrl = UART_HW_FLOWCTRL_DISABLE
    };
    uart_driver_install(UART_NUM, BUF_SIZE * 2, 0, 0, NULL, 0);
    uart_param_config(UART_NUM, &uart_config);
    uart_set_pin(UART_NUM, TXD_PIN, RXD_PIN, UART_PIN_NO_CHANGE, UART_PIN_NO_CHANGE);

    //sendSMS(phoneNumber, messageText);
}

void playAudio(const int32_t* data, size_t size) {
    size_t bytes_written;
    for (size_t i = 0; i < size; i++) {
        i2s_write(I2S_NUM, &data[i], sizeof(int32_t), &bytes_written, portMAX_DELAY);
    }
}

void loop() {
    // Relay data from Serial Monitor to M66
    while (Serial.available()) {
        size_t len = Serial.available();
        uint8_t data[len];
        Serial.readBytes(data, len);
        uart_write_bytes(UART_NUM, (const char *)data, len);

        // Echo the sent data back to Serial Monitor for confirmation
        Serial.print("> ");
        Serial.write(data, len);
    }

    // Relay data from M66 to Serial Monitor
    uint8_t data[UART_BUF_SIZE];
    int length = uart_read_bytes(UART_NUM, data, UART_BUF_SIZE, 20 / portTICK_RATE_MS);
    if (length > 0) {
        Serial.print("< ");
        Serial.write(data, length);
    }

    // Play beep every second
    unsigned long currentTime = millis();
    if (currentTime - lastTime > 4000) { // Every 1 second
        lastTime = currentTime;
        //playBeep();
        playAudio(audio_data, audio_data_len);
    }
}


void sendATCommand(const char* cmd) {
  uart_write_bytes(UART_NUM, cmd, strlen(cmd));
  uart_write_bytes(UART_NUM, "\r\n", 2); // Send newline character
  delay(1000); // Wait for command execution
}

void sendSMS(const char* number, const char* message) {
  sendATCommand("AT"); // Check if the module is responding
  sendATCommand("AT+CMGF=1"); // Set SMS to text mode

  char cmgsCmd[30];
  sprintf(cmgsCmd, "AT+CMGS=\"%s\"", number); // Prepare CMGS command
  sendATCommand(cmgsCmd);

  uart_write_bytes(UART_NUM, messageText, strlen(messageText)); // Message content
}

void playBeep() {
  i2s_set_clk(I2S_NUM, 8000, I2S_BITS_PER_SAMPLE_16BIT, I2S_CHANNEL_MONO);
    
    size_t bytes_written;
    for (int i = 0; i < sizeof(wavData) / sizeof(wavData[0]); i++) {
        const int16_t sample = (int16_t)(wavData[i] << 8);  // Convert 8-bit to 16-bit
        i2s_write(I2S_NUM, &sample, sizeof(sample), &bytes_written, portMAX_DELAY);
    }
}